function [theta,VC,ZZ,BETA] = VARestimation(Y,R,p)
%_____________________________________________________________________________________________________________________
% VARestimation estimates the subset model of a VAR(P) implied by the
% constraint matrix R.
%
%   PARAMETERS:
%   Y is an n-vector with the data of the dependent variable
%   R is the matrix of constraints
%   p is maximum number of lags allowed in the general VAR(p) model
%
%   theta is the vector of parameter estimates
%   VC is the associated robust variance covariance matrix
%   ZZ is the matrix of regressors, which is needed as input for forecasts
%_____________________________________________________________________________________________________________________
%
% 1. Initialize a few variables
%_____________________________________________________________________________________________________________________
[g,n]   = size(Y);   % g is the number of dependent variables
                     % n is the length of time series
YY      = [mean(Y,2)*ones(1,p),Y];   % Dependent variable; presample = unconditional mean
%
% Construct the Z matrix
YY      = YY(:,end:-1:1); % Invert the order of the time series
YY      = YY(:)*ones(1,n); % Vectorize the matrix of dependent variables and replicate it n times
Bidx    = (1:g*p)'*ones(1,n+1) + ones(g*p,1)*(0:g:g*n); % Construct the index of lags at each time t
ZZ      = YY(Bidx); % Define the Z matrix of lags
ZZ      = ZZ(:,end:-1:1); % Invert back the order of time series
ZZ      = [ones(1,size(ZZ,2)); ZZ]; % Add the first line of constants
Z       = ZZ(:,1:end-1); % Drop the last line, which corresponds to Z_n. Keep it in ZZ for later
%_____________________________________________________________________________________________________________________
%
% 2. Parameter estimation
%_____________________________________________________________________________________________________________________
y       = Y(:); % Vectorise the matrix of dependent variables
ZZ1     = Z*Z';
theta10 = kron(ZZ1,sparse(eye(g)));
theta11 = kron(Z,sparse(eye(g)));
theta1  = (R'*theta10*R)\(R'*theta11*y); % Estimate without VC matrix
b       = R*theta1;
BETA    = reshape(b,g,1+g*p);
BZ      = BETA*Z;
U       = Y-BZ;
S       = U*U'/n; % Residuals VC matrix
S1      = inv(S); % Inverse of the residual VC matrix
A1      = kron(ZZ1,S1);
A       = -R'*A1*R/n;
B1      = R'*kron(Z,S1);
theta   = n*(-A)\B1*y;
%_____________________________________________________________________________________________________________________
%
% 3. Robust variance covariance matrix estimation
%_____________________________________________________________________________________________________________________
if nargout >1
    u   = y-(theta11'*R)*theta;
    U   = reshape(u,g,n);
    % Create block diagonal matrix from the columns of U
    UD1 = mat2cell(U, size(U,1), ones(1,size(U,2)));
    UD  = sparse(blkdiag(UD1{:}));
    %
    B   = (B1*UD)*(UD'*B1')/n;
    VC  = A\B/A;
end