function [L,aJudgInSample,err,aJudg,yOut] = VARrisk(X,Y,h,htilde,atilde,alpha,p)
%_____________________________________________________________________________________________________________________
% Estimate risk of VAR(p) submodels
% X         = vector of scalars identifying the VAR(p) submodel to be estimated 
% Y         = (g,n) matrix of dependent variables
% h         = forecasting horizon
% htilde    = forecast horizon at which the judgmental decision is expressed
% atilde    = judgmental decision
% alpha     = confidence level needed for testing
% p         = maximum number of lags allowed in the general VAR(p) model
%_____________________________________________________________________________________________________________________
%
% 1. Initialize a few variables
%_____________________________________________________________________________________________________________________
%
[g,n] = size(Y);   % g is the number of dependent variables
                   % n is the length of time series
%
% Construct the restriction matrix R
X(X==1) = find(X==1);
X   = X(X>0);    % Drop the zeros
if isempty(X), X=1; end % Condition needed for patternsearch, in case it searches for X with only zeros.
%disp(X)
px  = length(X); % Number of regressors (including constants)
R   = zeros(g*(1+p*g), px);
idx = sub2ind(size(R), X, 1:px);
R(idx) = 1;
%
% Options for numerical optimization
MaxIter = 1000;         % MaxIterations
MaxFE   = 10000;        % MaxFunctionEvaluations
DSP     = 'off';        % Display
StepTol = 1e-15;        % StepTolerance
OptTol  = 1e-15;        % OptimalityTolerance
FunTol  = 1e-15;        % FunctionTolerance
OPTIONSfmincon = optimoptions(@fmincon, 'MaxIterations', MaxIter, 'MaxFunctionEvaluations', MaxFE, ...
    'GradObj','off', 'Hessian','off', 'Display', DSP, ...
    'StepTolerance', StepTol, 'OptimalityTolerance', OptTol, 'FunctionTolerance', FunTol,...
    'Algorithm','sqp','SpecifyObjectiveGradient',true,'SpecifyConstraintGradient',true);
%_____________________________________________________________________________________________________________________
%
% 2. Compute QMLE and VC
%_____________________________________________________________________________________________________________________
[theta,VC,Z,BETA] = VARestimation(Y,R,p);
% Determine the stability of the estimated VAR
XX = cell(1,p); 
for i=1:p
    XX(i) = mat2cell(BETA(:,1+g*(i-1)+1:1+g*i),g,g);
end
Mdl = varm('Constant', BETA(:,1), 'AR',XX);
ar = [{eye(g)} Mdl.AR];
Mdl = LagOp(ar);
Mdl = reflect(Mdl); % Negate all lags > 0
% If roots are unstable, discard the model
if ~isStable(Mdl)
    L = 1e10;
    aJudgInSample = zeros(n-h,1);
    err = aJudgInSample;
    aJudg = 99;
%    disp(X)
    return
end
%_____________________________________________________________________________________________________________________
%
% 3. Check if the judgmental decision is optimal
%_____________________________________________________________________________________________________________________
[Yf1,VCf] = VARforecast(Y,R,p,htilde,theta,VC,Z);
Yf        = mean(Yf1);
Tx        = sqrt(n)*(-Yf +atilde)/sqrt(VCf);
%_____________________________________________________________________________________________________________________
%
% 4. Compute constrained QMLE
%_____________________________________________________________________________________________________________________
objfungrad = @(theta)VARloglik(theta,Y,R,p);
lb = [ ]; ub = [ ];   % No upper or lower bounds
c_alpha   = norminv(alpha/2);
if Tx >= c_alpha && Tx <= -c_alpha
    confungrad = @(theta)VARconstraint(Y,R,p,htilde,theta,Z,atilde);
    thetacon  = fmincon(objfungrad,theta,[],[],[],[],lb,ub,confungrad,OPTIONSfmincon);
    Yf1       = VARforecast(Y,R,p,h,thetacon,VC,Z);
    aJudg     = mean(Yf1);
elseif Tx < c_alpha
    atildeminus = Yf + sqrt(VCf)*c_alpha/sqrt(n);
    confungrad = @(theta)VARconstraint(Y,R,p,htilde,theta,Z,atildeminus);
    thetaconminus = fmincon(objfungrad,theta,[],[],[],[],lb,ub,confungrad,OPTIONSfmincon);
    Yf1     = VARforecast(Y,R,p,h,thetaconminus,VC,Z);
    aJudg     = mean(Yf1);
elseif Tx > - c_alpha 
    atildeplus = Yf + sqrt(VCf)*(-c_alpha)/sqrt(n);
    confungrad = @(theta)VARconstraint(Y,R,p,htilde,theta,Z,atildeplus);
    thetaconplus = fmincon(objfungrad,theta,[],[],[],[],lb,ub,confungrad,OPTIONSfmincon);
    Yf1	 = VARforecast(Y,R,p,h,thetaconplus,VC,Z);
    aJudg = mean(Yf1);
end
%_____________________________________________________________________________________________________________________
%
% 5. Construct decision rule
%_____________________________________________________________________________________________________________________
aJudgInSample  = zeros(n-h,1); % Now h is the forecast of interest, which does not necessarily coincide with htilde
yOut           = aJudgInSample;
% Start the loop to compute the in sample decision
for t = 1:n-h
    if Tx >= c_alpha && Tx <= -c_alpha
        Yf  = VARforecast(Y(:,1:t),R,p,h,thetacon,VC,Z);     % In sample QMLE decision
        aJudgInSample(t) = mean(Yf);
    elseif Tx < c_alpha
        Yf = VARforecast(Y(:,1:t),R,p,h,thetaconminus,VC,Z); % In sample QMLE decision
        aJudgInSample(t) = mean(Yf);
    elseif Tx > -c_alpha
        Yf = VARforecast(Y(:,1:t),R,p,h,thetaconplus,VC,Z);  % In sample QMLE decision
        aJudgInSample(t) = mean(Yf);
    end
    yOut(t)              = mean(Y(1,t+1:t+h));               % Out of sample dependent variable to evaluate the performance of the decision
end
%_____________________________________________________________________________________________________________________
%
% 6. Compute the in sample loss of the optimal decision rule and the otpimal estimate
%_____________________________________________________________________________________________________________________
err = yOut - aJudgInSample; %figure, plot([yOut, aJudgInSample])
L = 0.5*(err'*err)/(n-h);
%disp(L)